#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "ntp.h"
#include <sys/socket.h>
#include <arpa/inet.h>

void init_req(struct ntp_msg* msg, struct timestamp *current) {
  memset(msg, 0, sizeof(struct ntp_msg));
  msg->version      = 3U; // version 3
  msg->mode         = 3U; // mode client
  msg->transmit_ts.sec = current->sec;
  msg->transmit_ts.fra = current->fra;
}

void ntp_msg_read(struct ntp_msg *dest, unsigned char* buf, int offset) {
        uint32_t delay, dispersion;
        dest->leap = buf[offset] >> 6;
        dest->version = (buf [offset] >> 3) & 0x5;
        dest->mode = buf[offset] & 0x5;
        dest->stratum = buf[offset + 1];
        dest->poll = buf[offset + 2];
        dest->precision = buf[offset + 3];
        memcpy(&delay, buf+offset+4, 4);
        dest->delay = ntohl(delay);
        memcpy(&dispersion, buf+offset+8, 4);
        dest->dispersion = ntohl(dispersion);
        memcpy(&dest->reference, buf+offset+12, 4);
        ts_read(&(dest->reference_ts), buf+offset+16);
        ts_read(&(dest->originate_ts), buf+offset+24);
        ts_read(&(dest->receive_ts),   buf+offset+32);
        ts_read(&(dest->transmit_ts),  buf+offset+40);
}

void ntp_msg_write(struct ntp_msg* src, unsigned char* buf, int offset) {
        uint32_t delay, dispersion;
        memset(buf+offset, 0, NTP_LEN);
        buf[offset]   = src->leap << 6 | src->version << 3 | src->mode;
        buf[offset+1] = src->stratum;
        buf[offset+2] = src->poll;
        buf[offset+3] = src->precision;
        delay = htonl(src->delay);
        memcpy(buf+offset+4, &delay, 4);
        dispersion = htonl(src->dispersion);
        memcpy(buf+offset+8, &dispersion, 4);
        memcpy(buf+offset+12, &src->reference, 4);
        ts_write(&(src->reference_ts), buf+offset+16);
        ts_write(&(src->originate_ts), buf+offset+24);
        ts_write(&(src->receive_ts),   buf+offset+32);
        ts_write(&(src->transmit_ts),  buf+offset+40);
}

void fprint_ntp_msg(FILE *f, struct ntp_msg* msg) {
  fprintf(f,"orig: %s\nrecv: %s\ntran: %s\nref: %s (id: %s)\n",
	  tstop(&(msg->originate_ts)),
	  tstop(&(msg->receive_ts)),
	  tstop(&(msg->transmit_ts)),
	  tstop(&(msg->reference_ts)),
	  inet_ntoa(msg->reference));
}
